// ============================================================================
// Licensed Materials - Property of IBM
// Project  Zero
//
// (C) Copyright IBM Corp. 2008  All Rights Reserved.
//
// US Government Users Restricted Rights - Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.
// ============================================================================
dojo.provide("zero.webtools.CodeViewer");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("dijit.form.Form");
dojo.require("dijit.form.TextBox");
dojo.require("dijit.form.Button");
dojo.require("dijit.Dialog");
dojo.require("dijit.Toolbar");
dojo.require("dijit.layout.BorderContainer");
dojo.require("dijit.layout.ContentPane");

dojo.declare(
    "zero.webtools.CodeViewer",
    [dijit._Widget, dijit._Templated],
	{
		widgetsInTemplate: true,
		
		templateString: "<div class='codeviewer'>" +
			"<div dojoType='dijit.layout.BorderContainer' dojoAttachPoint='mainContainer' style='width: 100%; height: 100%;'>" +
			"<div dojoType='dijit.layout.ContentPane' region='top' style='height:20px; '>" +
			"<span dojoAttachPoint='titleNode' class='title'>title</span></div>" +
			"<div dojoType='dijit.layout.ContentPane' dojoattachpoint='editorContentPane' region='center' style='border: 1px solid #848484; border-top: 0 none; overflow:auto;'>"+
			"<div class='lineNumbers' dojoAttachPoint='lineNode' style='position:absolute; top:0; left:0;'><div dojoAttachPoint='numbers'>1:</div></div>"+
			"<div class='codeviewerinner' dojoAttachPoint='editorNode' style='position:absolute; top:0; left:24px; margin-left:5px'></div>"+
			"</div></div>",
		
		editWidget : null,

        postCreate: function() {    			
        },
				
		_data: "",
		lineCount: 1,
		setData: function(data) {
			//dojox.highlight.init(this.contentNode);
			this._data = data;
			this.titleNode.innerHTML=this.fileName;
			this.editorNode.innerHTML = ''+this._escapeXml(this._data)+'';
			this.numbers.innerHTML='';
			this.lineCount = this._countLines(1,this.editorNode.childNodes);
			this.onLinesChanged(1,this.lineCount);
			this.onGotoLine(1);
		},
		_countLines: function(lineNumber, startNodes){
			if (!startNodes) {return lineNumber;}
			for (var i=0; (i < startNodes.length); i++) {
				if ((startNodes[i].nodeType == 1) && (startNodes[i].nodeName == "BR") &&
				    (startNodes[i].nextSibling)) {
					lineNumber = this._countLines(lineNumber+1, startNodes[i].childNodes);
					
				} else {
					lineNumber = this._countLines(lineNumber, startNodes[i].childNodes);									
				}
			}
			return lineNumber;
		},
		onPromptGotoLineNumber: function() {
			var dialog = this.gotoDialog;
			var widget = this;
			var execgoto = dojo.hitch(widget, widget.onGotoLine); 
			this.gotoForm.onExecute = function(){
				var line = 1*widget.gotoDialogLineNumber.getValue();
				if(line <= 0){
					return;
				}
				dialog.hide();
				execgoto(line);
			};
			dialog.show();
			setTimeout(dojo.hitch(this.gotoDialogLineNumber, this.gotoDialogLineNumber.focus),50);			
		},
		onGotoLine: function(lineNumber) {
			var body = this.editorContentPane.domNode;
			body.scrollTop = 13*(lineNumber-1);
		},
		onOpen: function() {
			var bindArgs = {
			    url: this.filePath,
		        load: dojo.hitch(this, function(data, args) { 
					this.onOpenComplete(true, data, args);
				}),
		        error: dojo.hitch(this, function(type, error) { 
					console.debug("Open Error: "+type+"/"+error);	
					this.onOpenComplete(false, type, error);
				}),
			    preventCache: true,
			    handleAs: "text"        
			};
		  	var request = dojo.xhrGet(bindArgs);								
		},		
		onOpenComplete: function(success, data, args) {		
			if (success){
				var contenttype = args.xhr.getResponseHeader("Content-Type");
				if (/(text\/.*|application\/(x-javascript|javascript|json|xml|ecmascript|html|.*\+xml))/i.test(contenttype)) {
					this.setData(data);
				} else {
					this.setData("Binary data (Content-Type:  "+contenttype+")");
				} 
				this.refresh();
			}
		},			
		onLinesChanged: function(start, end) {
			if (start == 0) start = 1;
			if (start<=end) {
				for (var i=start; i <= end; i++) {
					this.numbers.appendChild(document.createTextNode(i+":"));
					this.numbers.appendChild(document.createElement("br"));
				}
			} 
		},
		_escapeXml: function(/*String*/str, /*Boolean*/noSingleQuotes){
			//summary:
			//		Adds escape sequences for special characters in XML: &<>"
			str = str.replace(/&/gm, "&amp;").replace(/</gm, "&lt;").replace(/>/gm, "&gt;").replace(/"/gm, "&quot;").replace(/\n/gm, "<br/>").replace(/\t/gm, "    ");
			return str.replace(/\s/gm, "&nbsp;"); // string
		},	
		
		_dirtyView : false,

		refresh: function(size) {						
			// compute the line number width
			var width = 16;
			for (var i=this.lineCount; i >= 10; i=i/10) {
				width = width + 8;
				if(dojo.isIE){
					width = width + 1;
				}
			}
			this.lineNode.style.width = width;
			this.editorNode.style.left = width;
		}	
	}
);